@@ -40,6 +40,34 @@ import os.path
 
 import catnip.config as conf
 import iris.exceptions
+from dask import array as da
+
+
+def _get_xy_noborder(mask):
+    """
+    make  a function that returns the indices
+    of where the mask is valid. If the mask is all True (all masked)
+    raises a ValueError
+
+    args
+    ----
+    mask: mask from numpy array
+
+    Returns
+    -------
+    x1, x2, y1, y2: int giving space where the data is valid
+
+    """
+
+    if np.all(mask):
+        raise ValueError("All values masked - can't get indices")
+    ys, xs = np.where(~mask)
+    x1 = min(xs)
+    x2 = max(xs) + 1
+    y1 = min(ys)
+    y2 = max(ys) + 1
+
+    return x1, x2, y1, y2
 
 
 def add_aux_unrotated_coords(cube):
@@ -403,6 +431,71 @@ month_number, season, season_number, year
     return ccube
 
 
+def extract_rot_cube(cube, min_lat, min_lon, max_lat, max_lon):
+    """
+    Function etracts the specific region from the cube.
+    args
+    ----
+    cube: cube on rotated coord system, used as reference grid for transformation.
+    Returns
+    -------
+    min_lat: The minimum latitude point of the desired extracted cube.
+    min_lon: The minimum longitude point of the desired extracted cube.
+    max_lat: The maximum latitude point of the desired extracted cube.
+    max_lon: The maximum longitude point of the desired extracted cube.
+    An example:
+    >>> file = os.path.join(conf.DATA_DIR, 'rcm_monthly.pp')
+    >>> cube = iris.load_cube(file, 'air_temperature')
+    >>> min_lat = 50
+    >>> min_lon = -10
+    >>> max_lat = 60
+    >>> max_lon = 0
+    >>> extracted_cube = extract_rot_cube(cube, min_lat, min_lon, max_lat, max_lon)
+    >>> max_lat_cube =  np.max(extracted_cube.coord('latitude').points)
+    >>> print(f'{max_lat_cube:.3f}')
+    61.365
+    >>> min_lat_cube = np.min(extracted_cube.coord('latitude').points)
+    >>> print(f'{min_lat_cube:.3f}')
+    48.213
+    >>> max_lon_cube = np.max(extracted_cube.coord('longitude').points)
+    >>> print(f'{max_lon_cube:.3f}')
+    3.643
+    >>> min_lon_cube = np.min(extracted_cube.coord('longitude').points)
+    >>> print(f'{min_lon_cube:.3f}')
+    -16.292
+    """
+
+    # adding unrotated coords to the cube
+    cube = add_aux_unrotated_coords(cube)
+
+    # mask the cube using the true lat and lon
+    lats = cube.coord("latitude").points
+    lons = cube.coord("longitude").points
+    select_lons = (lons >= min_lon) & (lons <= max_lon)
+    select_lats = (lats >= min_lat) & (lats <= max_lat)
+    selection = select_lats & select_lons
+    selection = da.broadcast_to(selection, cube.shape)
+    cube.data = da.ma.masked_where(~selection, cube.core_data())
+
+    # grab a single 2D slice of X and Y and take the mask
+    lon_coord = cube.coord(axis="X", dim_coords=True)
+    lat_coord = cube.coord(axis="Y", dim_coords=True)
+    for yx_slice in cube.slices(["grid_latitude", "grid_longitude"]):
+        cmask = yx_slice.data.mask
+        break
+
+    # now cut the cube down along X and Y coords
+    x1, x2, y1, y2 = _get_xy_noborder(cmask)
+    idx = len(cube.shape) * [slice(None)]
+
+    idx[cube.coord_dims(cube.coord(axis="x", dim_coords=True))[0]] = slice(x1, x2, 1)
+    idx[cube.coord_dims(cube.coord(axis="y", dim_coords=True))[0]] = slice(y1, y2, 1)
+
+    extracted_cube = cube[tuple(idx)]
+
+    return extracted_cube
+
+
 def remove_forecast_coordinates(iris_cube):
     """A function to remove the forecast_period and
     forecast_reference_time coordinates from the UM PP files
